
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.043.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Linked Data Fragment RDF store",
  "dist_author" => [
    "Patrick Hochstenbach  C<< <patrick.hochstenbach\@ugent.be> >>"
  ],
  "dist_name" => "AtteanX-Store-LDF",
  "dist_version" => "0.005_03",
  "license" => "perl",
  "module_name" => "AtteanX::Store::LDF",
  "recursive_test_files" => 1,
  "requires" => {
    "Attean" => "0.010",
    "MooX::Log::Any" => 0,
    "RDF::LDF" => 0,
    "perl" => "v5.14.1"
  },
  "test_requires" => {
    "RDF::Generator::Void" => 0,
    "RDF::LinkedData" => "0.74",
    "Test::Exception" => 0,
    "Test::LWP::UserAgent" => 0,
    "Test::More" => "0.88",
    "Test::Roo" => 0
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "RDF::Generator::Void" => 0,
  "RDF::LinkedData" => "0.74",
  "Test::Exception" => 0,
  "Test::LWP::UserAgent" => 0,
  "Test::More" => "0.88",
  "Test::Roo" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
